import scala.actors._
import java.util.Random

abstract class NameResolver extends Actor {
  import java.net.{InetAddress, UnknownHostException}
  def getIp(name: String): Option[InetAddress] = {
    try {
      Some(InetAddress.getByName(name))
    } catch {
      case _:UnknownHostException => None
    }
  }
}

class NameResolverReact extends NameResolver {
  def act() {
    react {
      case (name: String, actor: Actor) =>
        actor ! getIp(name)
        act()
      case "EXIT" =>
        println("Name resolver exiting.")
        // quit
      case msg =>
        println("Unhandled message: "+ msg)
        act()
    }
  }
}

class NameResolverReceive extends NameResolver {
  var keep_going = true

  def act() {
    while (keep_going) {
      receive {
        case (name: String, actor: Actor) =>
          actor ! getIp(name)
        case "EXIT" =>
          println("Name resolver exiting.")
          keep_going = false
        case msg =>
          println("Unhandled message: "+ msg)
      }
    }
  }
}

object Test extends Actor {

  val generator : Random = new Random

  var resolvers : Array[NameResolver] = null
  var num       : Int = 0

  val sites = Array("www.cs.colorado.edu", "www.google.edu",
                    "cnet.com", "microsoft.com", "java.sun.com",
                    "daringfireball.net", "www.cnn.com", "uci.edu",
                    "me.com", "amazon.com")

  def act() {
    for (i <- 1 to 1000) {
      for (resolver <- resolvers) {
        resolver ! (sites(generator.nextInt(10)), this)
      }
    }
    for (i <- 1 to 1000*num) {
      this.receive {
         case msg => println(i + ":" + msg)
      }
    }
    println("Thread count: " + Thread.activeCount())
    for (resolver <- resolvers) {
      resolver ! "EXIT"
    }
    Thread.sleep(1000)
    println("Final Thread count: " + Thread.activeCount())

  }

  def main(args: Array[String]) {

    val kind : String = args(0)

    num = args(1).toInt

    resolvers = new Array[NameResolver](num)

    for (i <- 0 to num-1) {
      if (kind == "receive") {
        resolvers(i) = new NameResolverReceive
      } else {
        resolvers(i) = new NameResolverReact
      }
    }

    for (resolver <- resolvers) {
      resolver.start()
    }

    this.start()

  }
}
